@@ -20,7 +20,7 @@ declare(strict_types=1);
  *
  */
 
-namespace OCA\Spreed\Activity;
+namespace OCA\Talk\Activity;
 
 
 use OCP\Activity\ISetting;
