@@ -24,36 +24,44 @@ declare(strict_types=1);
  *
  */
 
-namespace OCA\Spreed\Controller;
+namespace OCA\Talk\Controller;
 
-use OCA\Spreed\Chat\ChatManager;
-use OCA\Spreed\Chat\MessageParser;
-use OCA\Spreed\Exceptions\InvalidPasswordException;
-use OCA\Spreed\Exceptions\ParticipantNotFoundException;
-use OCA\Spreed\Exceptions\RoomNotFoundException;
-use OCA\Spreed\Exceptions\UnauthorizedException;
-use OCA\Spreed\GuestManager;
-use OCA\Spreed\Manager;
-use OCA\Spreed\Participant;
-use OCA\Spreed\Room;
-use OCA\Spreed\TalkSession;
-use OCA\Spreed\Webinary;
+use OCA\Circles\Api\v1\Circles;
+use OCA\Circles\Model\Member;
+use OCA\Talk\Chat\ChatManager;
+use OCA\Talk\Chat\MessageParser;
+use OCA\Talk\Events\UserEvent;
+use OCA\Talk\Exceptions\InvalidPasswordException;
+use OCA\Talk\Exceptions\ParticipantNotFoundException;
+use OCA\Talk\Exceptions\RoomNotFoundException;
+use OCA\Talk\Exceptions\UnauthorizedException;
+use OCA\Talk\GuestManager;
+use OCA\Talk\Manager;
+use OCA\Talk\Participant;
+use OCA\Talk\Room;
+use OCA\Talk\TalkSession;
+use OCA\Talk\Webinary;
+use OCP\App\IAppManager;
 use OCP\AppFramework\Http;
 use OCP\AppFramework\Http\DataResponse;
 use OCP\AppFramework\Utility\ITimeFactory;
 use OCP\Comments\IComment;
+use OCP\EventDispatcher\IEventDispatcher;
 use OCP\IL10N;
 use OCP\IRequest;
 use OCP\IUser;
 use OCP\IUserManager;
 use OCP\IGroup;
 use OCP\IGroupManager;
-use Symfony\Component\EventDispatcher\EventDispatcherInterface;
-use Symfony\Component\EventDispatcher\GenericEvent;
 
 class RoomController extends AEnvironmentAwareController {
+
+	public const EVENT_BEFORE_ROOMS_GET = self::class . '::preGetRooms';
+
 	/** @var string|null */
 	private $userId;
+	/** @var IAppManager */
+	private $appManager;
 	/** @var TalkSession */
 	private $session;
 	/** @var IUserManager */
@@ -66,7 +74,7 @@ class RoomController extends AEnvironmentAwareController {
 	private $guestManager;
 	/** @var ChatManager */
 	private $chatManager;
-	/** @var EventDispatcherInterface */
+	/** @var IEventDispatcher */
 	private $dispatcher;
 	/** @var MessageParser */
 	private $messageParser;
@@ -78,18 +86,20 @@ class RoomController extends AEnvironmentAwareController {
 	public function __construct(string $appName,
 								?string $UserId,
 								IRequest $request,
+								IAppManager $appManager,
 								TalkSession $session,
 								IUserManager $userManager,
 								IGroupManager $groupManager,
 								Manager $manager,
 								GuestManager $guestManager,
 								ChatManager $chatManager,
-								EventDispatcherInterface $dispatcher,
+								IEventDispatcher $dispatcher,
 								MessageParser $messageParser,
 								ITimeFactory $timeFactory,
 								IL10N $l10n) {
 		parent::__construct($appName, $request);
 		$this->session = $session;
+		$this->appManager = $appManager;
 		$this->userId = $UserId;
 		$this->userManager = $userManager;
 		$this->groupManager = $groupManager;
@@ -110,9 +120,8 @@ class RoomController extends AEnvironmentAwareController {
 	 * @return DataResponse
 	 */
 	public function getRooms(): DataResponse {
-		$this->dispatcher->dispatch(self::class . '::preGetRooms', new GenericEvent(null, [
-			'userId' => $this->userId,
-		]));
+		$event = new UserEvent($this->userId);
+		$this->dispatcher->dispatch(self::EVENT_BEFORE_ROOMS_GET, $event);
 
 		$rooms = $this->manager->getRoomsForParticipant($this->userId, true);
 
@@ -338,17 +347,7 @@ class RoomController extends AEnvironmentAwareController {
 			return [];
 		}
 
-		return [
-			'id' => (int) $lastMessage->getId(),
-			'actorType' => $message->getActorType(),
-			'actorId' => $message->getActorId(),
-			'actorDisplayName' => $message->getActorDisplayName(),
-			'timestamp' => $lastMessage->getCreationDateTime()->getTimestamp(),
-			'message' => $message->getMessage(),
-			'messageParameters' => $message->getMessageParameters(),
-			'systemMessage' => $message->getMessageType() === 'system' ? $message->getMessageRaw() : '',
-			'messageType' => $message->getMessageType(),
-		];
+		return $message->toArray();
 	}
 
 	/**
@@ -359,9 +358,10 @@ class RoomController extends AEnvironmentAwareController {
 	 * @param int $roomType
 	 * @param string $invite
 	 * @param string $roomName
+	 * @param string $source
 	 * @return DataResponse
 	 */
-	public function createRoom(int $roomType, string $invite = '', string $roomName = ''): DataResponse {
+	public function createRoom(int $roomType, string $invite = '', string $roomName = '', string $source = ''): DataResponse {
 		switch ($roomType) {
 			case Room::ONE_TO_ONE_CALL:
 				return $this->createOneToOneRoom($invite);
@@ -369,6 +369,9 @@ class RoomController extends AEnvironmentAwareController {
 				if ($invite === '') {
 					return $this->createEmptyRoom($roomName, false);
 				}
+				if ($source === 'circles') {
+					return $this->createCircleRoom($invite);
+				}
 				return $this->createGroupRoom($invite);
 			case Room::PUBLIC_CALL:
 				return $this->createEmptyRoom($roomName);
@@ -404,7 +407,7 @@ class RoomController extends AEnvironmentAwareController {
 		try {
 			$room = $this->manager->getOne2OneRoom($this->userId, $targetUser->getUID());
 			$room->ensureOneToOneRoomIsFilled();
-			return new DataResponse(['token' => $room->getToken()], Http::STATUS_OK);
+			return new DataResponse($this->formatRoom($room, $room->getParticipant($currentUser->getUID())), Http::STATUS_OK);
 		} catch (RoomNotFoundException $e) {
 			$room = $this->manager->createOne2OneRoom();
 			$room->addUsers([
@@ -415,11 +418,7 @@ class RoomController extends AEnvironmentAwareController {
 				'participantType' => Participant::OWNER,
 			]);
 
-			return new DataResponse([
-				'token' => $room->getToken(),
-				'name' => $room->getName(),
-				'displayName' => $room->getDisplayName($currentUser->getUID()),
-			], Http::STATUS_CREATED);
+			return new DataResponse($this->formatRoom($room, $room->getParticipant($currentUser->getUID())), Http::STATUS_CREATED);
 		}
 	}
 
@@ -465,11 +464,67 @@ class RoomController extends AEnvironmentAwareController {
 
 		\call_user_func_array([$room, 'addUsers'], $participants);
 
-		return new DataResponse([
-			'token' => $room->getToken(),
-			'name' => $room->getName(),
-			'displayName' => $room->getDisplayName($currentUser->getUID()),
-		], Http::STATUS_CREATED);
+		return new DataResponse($this->formatRoom($room, $room->getParticipant($currentUser->getUID())), Http::STATUS_CREATED);
+	}
+
+	/**
+	 * Initiates a group video call from the selected circle
+	 *
+	 * @NoAdminRequired
+	 *
+	 * @param string $targetCircleId
+	 * @return DataResponse
+	 */
+	protected function createCircleRoom(string $targetCircleId): DataResponse {
+		if (!$this->appManager->isEnabledForUser('circles')) {
+			return new DataResponse([], Http::STATUS_BAD_REQUEST);
+		}
+
+		/** @var Circles $circlesApi */
+		try {
+			$circle = Circles::detailsCircle($targetCircleId);
+		} catch (\Exception $e) {
+			return new DataResponse([], Http::STATUS_NOT_FOUND);
+		}
+		$currentUser = $this->userManager->get($this->userId);
+
+		if (!$currentUser instanceof IUser) {
+			return new DataResponse([], Http::STATUS_NOT_FOUND);
+		}
+
+		// Create the room
+		$room = $this->manager->createGroupRoom($circle->getName());
+		$room->addUsers([
+			'userId' => $currentUser->getUID(),
+			'participantType' => Participant::OWNER,
+		]);
+
+		$participants = [];
+		foreach ($circle->getMembers() as $member) {
+			/** @var Member $member */
+			if ($member->getType() === Member::TYPE_USER || $member->getUserId() === '') {
+				// Not a user?
+				continue;
+			}
+
+			if ($currentUser->getUID() === $member->getUserId()) {
+				// Current user is already added
+				continue;
+			}
+
+			if ($member->getStatus() !== Member::STATUS_INVITED && $member->getStatus() !== Member::STATUS_MEMBER) {
+				// Only allow invited and regular members
+				continue;
+			}
+
+			$participants[] = [
+				'userId' => $member->getUserId(),
+			];
+		}
+
+		\call_user_func_array([$room, 'addUsers'], $participants);
+
+		return new DataResponse($this->formatRoom($room, $room->getParticipant($currentUser->getUID())), Http::STATUS_CREATED);
 	}
 
 	/**
@@ -502,11 +557,7 @@ class RoomController extends AEnvironmentAwareController {
 			'participantType' => Participant::OWNER,
 		]);
 
-		return new DataResponse([
-			'token' => $room->getToken(),
-			'name' => $room->getName(),
-			'displayName' => $room->getDisplayName($currentUser->getUID()),
-		], Http::STATUS_CREATED);
+		return new DataResponse($this->formatRoom($room, $room->getParticipant($currentUser->getUID())), Http::STATUS_CREATED);
 	}
 
 	/**
@@ -679,10 +730,49 @@ class RoomController extends AEnvironmentAwareController {
 				return new DataResponse([]);
 			}
 
+			\call_user_func_array([$this->room, 'addUsers'], $participantsToAdd);
+		} else if ($source === 'circles') {
+			if (!$this->appManager->isEnabledForUser('circles')) {
+				return new DataResponse([], Http::STATUS_BAD_REQUEST);
+			}
+
+			/** @var Circles $circlesApi */
+			try {
+				$circle = Circles::detailsCircle($newParticipant);
+			} catch (\Exception $e) {
+				return new DataResponse([], Http::STATUS_NOT_FOUND);
+			}
+
+			$participants = [];
+			foreach ($circle->getMembers() as $member) {
+				/** @var Member $member */
+				if ($member->getType() === Member::TYPE_USER || $member->getUserId() === '') {
+					// Not a user?
+					continue;
+				}
+
+				if (\in_array($member->getUserId(), $participants, true)) {
+					continue;
+				}
+
+				if ($member->getStatus() !== Member::STATUS_INVITED && $member->getStatus() !== Member::STATUS_MEMBER) {
+					// Only allow invited and regular members
+					continue;
+				}
+
+				$participantsToAdd[] = [
+					'userId' => $member->getUserId(),
+				];
+			}
+
+			if (empty($participantsToAdd)) {
+				return new DataResponse([]);
+			}
+
 			\call_user_func_array([$this->room, 'addUsers'], $participantsToAdd);
 		} else if ($source === 'emails') {
 			$data = [];
-			if ($this->room->changeType(Room::PUBLIC_CALL)) {
+			if ($this->room->setType(Room::PUBLIC_CALL)) {
 				$data = ['type' => $this->room->getType()];
 			}
 
@@ -696,34 +786,6 @@ class RoomController extends AEnvironmentAwareController {
 		return new DataResponse();
 	}
 
-	/**
-	 * @PublicPage
-	 * @RequireLoggedInModeratorParticipant
-	 *
-	 * @param string $newParticipant
-	 * @return DataResponse
-	 */
-	public function inviteEmailToRoom(string $newParticipant): DataResponse {
-		$currentUser = $this->userManager->get($this->userId);
-		if (!$currentUser instanceof IUser) {
-			return new DataResponse([], Http::STATUS_NOT_FOUND);
-		}
-
-		if ($this->room->getType() === Room::ONE_TO_ONE_CALL) {
-			return new DataResponse([], Http::STATUS_BAD_REQUEST);
-		}
-
-		$data = [];
-		// In case a guest is added to a non-public call, we change the call to a public call
-		if ($this->room->changeType(Room::PUBLIC_CALL)) {
-			$data = ['type' => $this->room->getType()];
-		}
-
-		$this->guestManager->inviteByEmail($this->room, $newParticipant);
-
-		return new DataResponse($data);
-	}
-
 	/**
 	 * @PublicPage
 	 * @RequireParticipant
@@ -830,7 +892,7 @@ class RoomController extends AEnvironmentAwareController {
 	 * @return DataResponse
 	 */
 	public function makePublic(): DataResponse {
-		if (!$this->room->changeType(Room::PUBLIC_CALL)) {
+		if (!$this->room->setType(Room::PUBLIC_CALL)) {
 			return new DataResponse([], Http::STATUS_BAD_REQUEST);
 		}
 
@@ -844,7 +906,7 @@ class RoomController extends AEnvironmentAwareController {
 	 * @return DataResponse
 	 */
 	public function makePrivate(): DataResponse {
-		if (!$this->room->changeType(Room::GROUP_CALL)) {
+		if (!$this->room->setType(Room::GROUP_CALL)) {
 			return new DataResponse([], Http::STATUS_BAD_REQUEST);
 		}
 
@@ -974,7 +1036,7 @@ class RoomController extends AEnvironmentAwareController {
 			return new DataResponse([], Http::STATUS_BAD_REQUEST);
 		}
 
-		$room->setParticipantType($participant, Participant::MODERATOR);
+		$room->setParticipantType($targetParticipant, Participant::MODERATOR);
 
 		return new DataResponse();
 	}
@@ -990,7 +1052,7 @@ class RoomController extends AEnvironmentAwareController {
 			return new DataResponse([], Http::STATUS_BAD_REQUEST);
 		}
 
-		$room->setParticipantTypeBySession($targetParticipant, Participant::GUEST_MODERATOR);
+		$room->setParticipantType($targetParticipant, Participant::GUEST_MODERATOR);
 
 		return new DataResponse();
 	}
@@ -1026,7 +1088,7 @@ class RoomController extends AEnvironmentAwareController {
 			return new DataResponse([], Http::STATUS_BAD_REQUEST);
 		}
 
-		$room->setParticipantType($participant, Participant::USER);
+		$room->setParticipantType($targetParticipant, Participant::USER);
 
 		return new DataResponse();
 	}
@@ -1046,7 +1108,7 @@ class RoomController extends AEnvironmentAwareController {
 			return new DataResponse([], Http::STATUS_BAD_REQUEST);
 		}
 
-		$room->setParticipantTypeBySession($targetParticipant, Participant::GUEST);
+		$room->setParticipantType($targetParticipant, Participant::GUEST);
 
 		return new DataResponse();
 	}
