@@ -20,12 +20,15 @@ declare(strict_types=1);
  *
  */
 
-namespace OCA\Spreed;
+namespace OCA\Talk;
 
 
-use OCA\Spreed\Exceptions\ParticipantNotFoundException;
+use OCA\Talk\Events\AddEmailEvent;
+use OCA\Talk\Events\ModifyParticipantEvent;
+use OCA\Talk\Exceptions\ParticipantNotFoundException;
 use OCP\DB\QueryBuilder\IQueryBuilder;
 use OCP\Defaults;
+use OCP\EventDispatcher\IEventDispatcher;
 use OCP\IDBConnection;
 use OCP\IL10N;
 use OCP\IURLGenerator;
@@ -33,11 +36,13 @@ use OCP\IUser;
 use OCP\IUserSession;
 use OCP\Mail\IMailer;
 use OCP\Util;
-use Symfony\Component\EventDispatcher\EventDispatcherInterface;
-use Symfony\Component\EventDispatcher\GenericEvent;
 
 class GuestManager {
 
+	public const EVENT_BEFORE_EMAIL_INVITE = self::class . '::preInviteByEmail';
+	public const EVENT_AFTER_EMAIL_INVITE = self::class . '::postInviteByEmail';
+	public const EVENT_AFTER_NAME_UPDATE = self::class . '::updateName';
+
 	/** @var IDBConnection */
 	protected $connection;
 
@@ -56,7 +61,7 @@ class GuestManager {
 	/** @var IL10N */
 	protected $l;
 
-	/** @var EventDispatcherInterface */
+	/** @var IEventDispatcher */
 	protected $dispatcher;
 
 	public function __construct(IDBConnection $connection,
@@ -65,7 +70,7 @@ class GuestManager {
 								IUserSession $userSession,
 								IURLGenerator $url,
 								IL10N $l,
-								EventDispatcherInterface $dispatcher) {
+								IEventDispatcher $dispatcher) {
 		$this->connection = $connection;
 		$this->mailer = $mailer;
 		$this->defaults = $defaults;
@@ -77,12 +82,12 @@ class GuestManager {
 
 	/**
 	 * @param Room $room
-	 * @param string $sessionId
+	 * @param Participant $participant
 	 * @param string $displayName
 	 * @throws \Doctrine\DBAL\DBALException
 	 */
-	public function updateName(Room $room, string $sessionId, string $displayName): void {
-		$sessionHash = sha1($sessionId);
+	public function updateName(Room $room, Participant $participant, string $displayName): void {
+		$sessionHash = sha1($participant->getSessionId());
 		$dispatchEvent = true;
 
 		try {
@@ -104,12 +109,9 @@ class GuestManager {
 			], ['session_hash']);
 		}
 
-
 		if ($dispatchEvent) {
-			$this->dispatcher->dispatch(self::class . '::updateName', new GenericEvent($room, [
-				'sessionId' => $sessionId,
-				'newName' => $displayName,
-			]));
+			$event = new ModifyParticipantEvent($room, $participant, 'name', $displayName);
+			$this->dispatcher->dispatch(self::EVENT_AFTER_NAME_UPDATE, $event);
 		}
 	}
 
@@ -128,7 +130,7 @@ class GuestManager {
 		$row = $result->fetch();
 		$result->closeCursor();
 
-		if (isset($row['display_name'])) {
+		if (isset($row['display_name']) && $row['display_name'] !== '') {
 			return $row['display_name'];
 		}
 
@@ -150,6 +152,10 @@ class GuestManager {
 		$map = [];
 
 		while ($row = $result->fetch()) {
+			if ($row['display_name'] === '') {
+				continue;
+			}
+
 			$map[$row['session_hash']] = $row['display_name'];
 		}
 		$result->closeCursor();
@@ -158,9 +164,8 @@ class GuestManager {
 	}
 
 	public function inviteByEmail(Room $room, string $email): void {
-		$this->dispatcher->dispatch(self::class . '::preInviteByEmail', new GenericEvent($room, [
-			'email' => $email,
-		]));
+		$event = new AddEmailEvent($room, $email);
+		$this->dispatcher->dispatch(self::EVENT_BEFORE_EMAIL_INVITE, $event);
 
 		$link = $this->url->linkToRouteAbsolute('spreed.pagecontroller.showCall', ['token' => $room->getToken()]);
 
@@ -203,9 +208,7 @@ class GuestManager {
 		try {
 			$this->mailer->send($message);
 
-			$this->dispatcher->dispatch(self::class . '::postInviteByEmail', new GenericEvent($room, [
-				'email' => $email,
-			]));
+			$this->dispatcher->dispatch(self::EVENT_AFTER_EMAIL_INVITE, $event);
 		} catch (\Exception $e) {
 		}
 	}
